Resume support for low-level uboot code, Version 5

Signed-off-by: Ben Dooks <ben-linux@fluff.org>

Index: u-boot/cpu/arm920t/start.S
===================================================================
--- u-boot.orig/cpu/arm920t/start.S	2007-02-21 22:52:55.000000000 +0100
+++ u-boot/cpu/arm920t/start.S	2007-02-21 23:01:54.000000000 +0100
@@ -150,11 +150,28 @@
 	str	r1, [r0]
 # endif
 
+#define MPLLCON		0x4c000004
+#define MPLLCFG		((0x90 << 12) + (0x2 << 4) + 0x2)
+	ldr	r0, =MPLLCON
+	ldr	r1, =MPLLCFG
+	str	r1, [r0]
+
 	/* FCLK:HCLK:PCLK = 1:2:4 */
-	/* default FCLK is 120 MHz ! */
+	/* default FCLK is 202 MHz ! */
 	ldr	r0, =CLKDIVN
 	mov	r1, #3
 	str	r1, [r0]
+
+	/* take sdram out of power down */
+	ldr	r0, =0x56000080		/* misccr */
+	ldr	r1, [ r0 ]
+	bic	r1, r1, #(S3C2410_MISCCR_nEN_SCLK0 | S3C2410_MISCCR_nEN_SCLK1 | S3C2410_MISCCR_nEN_SCLKE)
+	str	r1, [ r0 ]
+
+	/* ensure signals stabalise */
+	mov	r1, #128
+1:	subs	r1, r1, #1
+	bpl	1b
 #endif	/* CONFIG_S3C2400 || CONFIG_S3C2410 */
 
 	/*
@@ -163,7 +180,23 @@
 	 */
 #ifndef CONFIG_SKIP_LOWLEVEL_INIT
 	bl	cpu_init_crit
-#endif
+
+#if defined(CONFIG_S3C2410)
+	/* ensure some refresh has happened */
+	mov	r1, #4096
+1:	subs	r1, r1, #1
+	bpl	1b
+	
+	/* test for resume */
+	ldr	r1, =0x560000B4		/* gstatus2 */
+	ldr	r0, [ r1 ]
+	tst	r0, #0x02		/* is this resume from power down */
+	beq	not_resuming
+
+	ldr	pc, [r1, #4]		/* gstatus3 */
+not_resuming:
+#endif /* CONFIG_S3C2410 */
+#endif /* CONFIG_SKIP_LOWLEVEL_INIT */
 
 #ifndef CONFIG_SKIP_RELOCATE_UBOOT
 #ifndef CONFIG_S3C2410_NAND_BOOT
